package com.fjhx.customer.service.xiaoman.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.customer.contants.XiaomanContant;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import com.fjhx.customer.entity.customer.po.CustomerUser;
import com.fjhx.customer.entity.xiaoman.dto.XiaomanCustomerSelectDto;
import com.fjhx.customer.entity.xiaoman.po.XiaomanConfig;
import com.fjhx.customer.entity.xiaoman.po.XiaomanCustomer;
import com.fjhx.customer.entity.xiaoman.vo.CustomerListApiVo;
import com.fjhx.customer.entity.xiaoman.vo.XiaomanCustomerVo;
import com.fjhx.customer.entity.xiaoman.vo.XiaomanUpdateInfoVO;
import com.fjhx.customer.handle.HandleXiaomanData;
import com.fjhx.customer.service.xiaoman.XiaomanApiService;
import com.fjhx.customer.service.xiaoman.XiaomanConfigService;
import com.fjhx.customer.service.xiaoman.XiaomanCustomerService;
import com.fjhx.tenant.entity.dict.dto.DictTenantDataSelectDto;
import com.fjhx.tenant.entity.dict.po.DictTenantData;
import com.fjhx.tenant.entity.dict.vo.DictTenantDataVo;
import com.fjhx.tenant.service.dict.DictTenantDataService;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class XiaomanApiServiceImpl implements XiaomanApiService {
    @Resource
    XiaomanConfigService xiaomanConfigService;
    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Resource
    XiaomanCustomerService xiaomanCustomerService;
    @Resource
    DictTenantDataService dictTenantDataService;
    @Resource
    RedisCache redisCache;

    private static final Map<String, String> SELECTOR_KEY = new HashMap() {{
        put("trail_status", "customer_status");
        put("origin", "customer_source");
    }};

    public static void updateInfoData(String token, String body) {
        String res = HttpUtil.createPost(XiaomanContant.UPDATE_INFO_API_URL).header("Authorization", "Bearer " + token).body(body).execute().body();
        JSONObject jsonObject = JSONObject.parseObject(res);
        String error = jsonObject.getString("error");
        if (StrUtil.isNotBlank(error)) {
            throw new RuntimeException("获取小满数据异常：" + error);
        }
    }

    //    @DSTransactional
    @Override
    @Async
    public void initAllList() {
//        boolean result = setNxWithExpiration(XiaomanContant.TASK_STATUS_KEY, XiaomanContant.TASK_STATUS_VALUE, XiaomanContant.DELAY_DELETE_KEY_TIME);
//        if (!result) {
//            String mes = stringRedisTemplate.opsForValue().get(XiaomanContant.TASK_STATUS_DESC_KEY);
//            throw new RuntimeException(StrUtil.isBlank(mes)? "全量更新正在进行中" : mes);
//        }
//        stringRedisTemplate.opsForValue().set(XiaomanContant.TASK_STATUS_DESC_KEY, "全量更新正在进行中");
        try {
            getSelectorData();


            List<XiaomanCustomerVo> list = xiaomanCustomerService.getList(new XiaomanCustomerSelectDto());
            Set<Long> collect = list.stream().map(XiaomanCustomerVo::getCompanyId).collect(Collectors.toSet());
            int pageIndex = redisCache.getCacheObject(XiaomanContant.ALL_CUSTOMER_PAGE_INDEX_KEY);
            if (!NumberUtil.isValidNumber(pageIndex)) {
                pageIndex = 1;
            }
            int totalPage;
            do {
                XiaomanConfig config = xiaomanConfigService.getCache();
                String token = config.getAccessToken();

                String str = getData(XiaomanContant.ALL_CUSTOMER_API_URL, token, initPageParams(pageIndex));
                CustomerListApiVo customerListApiVo = HandleXiaomanData.handleAllCustomer(str, collect, true);
                int totalItem = customerListApiVo.getTotalItem();
                totalPage = (totalItem / XiaomanContant.PAGE_SIZE) + (totalItem % XiaomanContant.PAGE_SIZE > 0 ? 1 : 0);
                pageIndex++;
                redisCache.setCacheObject(XiaomanContant.ALL_CUSTOMER_PAGE_INDEX_KEY, pageIndex);
            } while (pageIndex <= totalPage);
        } catch (Exception e) {
            e.printStackTrace();
//            stringRedisTemplate.delete(XiaomanContant.TASK_STATUS_KEY);
//            stringRedisTemplate.opsForValue().set(XiaomanContant.TASK_STATUS_DESC_KEY, "全量更新出现异常，已停止");
//            return;
        }
//        stringRedisTemplate.delete(XiaomanContant.TASK_STATUS_KEY);
//        stringRedisTemplate.opsForValue().set(XiaomanContant.TASK_STATUS_DESC_KEY, "全量更新完成，完成时间：" + DateUtil.now());
    }

    /**
     * 设置key-value并设置过期时间
     *
     * @param key
     * @param value
     * @param seconds
     * @return boolean
     * @author hj
     * @date 2024/4/6 21:14
     */
    @Override
    public boolean setNxWithExpiration(String key, String value, long seconds) {
        Boolean result = stringRedisTemplate.opsForValue().setIfAbsent(key, value);
        if (result != null && result) {
            // 设置过期时间
            stringRedisTemplate.expire(key, seconds, TimeUnit.SECONDS);
            return true;
        }
        return false;
    }

    /**
     * 获取客户详情
     *
     * @param companyId
     * @author hj
     * @date 2024/4/6 21:38
     */
    @Override
    public String getCustomerDetail(Long companyId) {
        XiaomanConfig config = xiaomanConfigService.getCache();
        String token = config.getAccessToken();
        Map<String, Object> params = new HashMap<>();
        params.put("company_id", companyId);
        params.put("format", "1");
        return getData(XiaomanContant.CUSTOMER_DETAIL_API_URL, token, params);
    }

    /**
     * 客户更新列表
     *
     * @author hj
     * @date 2024/4/6 21:54
     */
    @Override
    @Async
    public void updateList(int hours) {
        try {
            //先更新一下数据字典
            getSelectorData();

            List<XiaomanCustomerVo> list = xiaomanCustomerService.getList(new XiaomanCustomerSelectDto());
            Set<Long> collect = list.stream().map(XiaomanCustomerVo::getCompanyId).collect(Collectors.toSet());
            int pageIndex = 1;
            int totalPage;
            do {
                XiaomanConfig config = xiaomanConfigService.getCache();
                String token = config.getAccessToken();

                Map<String, Object> params = initPageParams(pageIndex);
                String nowStr = DateUtil.offsetHour(DateUtil.date(), 1).toString("yyyy-MM-dd HH:00:00");
                params.put("start_time", DateUtil.parse(nowStr).offset(DateField.HOUR, -hours).toString("yyyy-MM-dd HH:00:00"));
                params.put("end_time", nowStr);
                String str = getData(XiaomanContant.GET_UPDATE_CUSTOMER_API_URL, token, params);
                CustomerListApiVo customerListApiVo = HandleXiaomanData.handleAllCustomer(str, collect, false);
                int totalItem = customerListApiVo.getTotalItem();
                totalPage = (totalItem / XiaomanContant.PAGE_SIZE) + (totalItem % XiaomanContant.PAGE_SIZE > 0 ? 1 : 0);
                pageIndex++;
            } while (pageIndex <= totalPage);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Override
    public void updateSingle(Long companyId) {
        //先更新一下数据字典
        getSelectorData();
        List<XiaomanCustomerVo> list = xiaomanCustomerService.getList(new XiaomanCustomerSelectDto());
        XiaomanCustomerVo detail = xiaomanCustomerService.detail(companyId);
        XiaomanCustomer customer = BeanUtil.copyProperties(detail, XiaomanCustomer.class);
        xiaomanCustomerService.processCustomer(customer, true, false);
    }

    private static Map<String, Object> initPageParams(int page) {
        Map<String, Object> params = new HashMap<>();
        params.put("start_index", page);
        params.put("count", XiaomanContant.PAGE_SIZE);
        return params;
    }


    private static String getData(String url, String token, Map<String, Object> params) {
        String res = HttpUtil.createGet(url).header("Authorization", "Bearer " + token).form(params).execute().body();
        JSONObject jsonObject = JSONObject.parseObject(res);
        String error = jsonObject.getString("error");
        if (StrUtil.isNotBlank(error)) {
            throw new RuntimeException("获取小满数据异常：" + error);
        }
        return res;
    }

    /**
     * 获取下拉框字典数据
     *
     * @author hj
     * @date 2024/4/7 10:51
     */
    @DSTransactional
    @Override
    public void getSelectorData() {
        XiaomanConfig config = xiaomanConfigService.getCache();
        String token = config.getAccessToken();
        Map<String, Object> params = new HashMap<>();
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId(XiaomanContant.TENANT_ID);
        SELECTOR_KEY.forEach((k, v) -> {
            params.put("field", k);
            String str = getData(XiaomanContant.SELECTOR_API_URL, token, params);
            JSONObject jsonObject = JSONObject.parseObject(str);
            DictTenantDataSelectDto dictTenantDataSelectDto = new DictTenantDataSelectDto();
            dictTenantDataSelectDto.setDictCode(v);
            List<DictTenantDataVo> dictTenantDataServiceList = dictTenantDataService.getList(dictTenantDataSelectDto);
            Map<String, DictTenantData> collect = dictTenantDataServiceList.stream().collect(Collectors.toMap(DictTenantDataVo::getDictValue, o -> BeanUtil.copyProperties(o, DictTenantData.class)));
            JSONArray data = jsonObject.getJSONArray("data");
            if ("trail_status".equals(k)) {
                dealCustomerStatusDict(data, collect);
            } else if ("origin".equals(k)) {
                dealCustomerSourceDict(data, collect);
            }
            log.info("获取小满下拉框字典数据：{}", jsonObject);
        });
        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.poll();
    }

    @Override
    public void updateXiaomanData(XiaomanUpdateInfoVO xiaomanUpdateInfoVO) {
        if (ObjectUtil.isNull(xiaomanUpdateInfoVO)) {
            throw new RuntimeException("数据为空");
        }
        if (StrUtil.isBlank(xiaomanUpdateInfoVO.getName())) {
            throw new RuntimeException("客户名称为空");
        }
        xiaomanUpdateInfoVO.setPoolId(0);
//        List<XiaomanUpdateInfoVO.Customer> customers = xiaomanUpdateInfoVO.getCustomers();
//        if (CollectionUtil.isEmpty(customers)){
//            throw new RuntimeException("客户联系人为空");
//        }
//        for (XiaomanUpdateInfoVO.Customer customer : customers) {
//            if (StrUtil.isBlank(customer.getName())){
//                throw new RuntimeException("客户联系人名称存在为空");
//            }
//            if (StrUtil.isBlank(customer.getEmail())){
//                throw new RuntimeException("客户联系人email存在为空");
//            }
//            customer.setMainCustomerFlag("0");
//        }
//        xiaomanUpdateInfoVO.setCustomers(customers);

        String body = JSON.toJSONString(xiaomanUpdateInfoVO);
        XiaomanConfig config = xiaomanConfigService.getCache();
        String token = config.getAccessToken();
        updateInfoData(token, body);
    }

    private void dealCustomerSourceDict(JSONArray data, Map<String, DictTenantData> collect) {
        List<DictTenantData> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject jsonObject = data.getJSONObject(i);
            String label = jsonObject.getString("origin_name");
            String key = jsonObject.getString("origin_id");
            boolean result = collect.containsKey(label);
            if (!result) {
                DictTenantData dictTenantData = new DictTenantData();
                dictTenantData.setDictCode("customer_source");
                dictTenantData.setDictValue(label);
                dictTenantData.setDictKey(key);
                dictTenantData.setTenantId(XiaomanContant.TENANT_ID);
                dictTenantData.setSort(100);
                list.add(dictTenantData);
            } else {
                DictTenantData dictTenantData = collect.get(label);
                dictTenantData.setDictKey(key);
                dictTenantDataService.updateById(dictTenantData);
            }
        }
        if (!list.isEmpty()) {
            dictTenantDataService.saveBatch(list);
        }
    }

    private void dealCustomerStatusDict(JSONArray data, Map<String, DictTenantData> collect) {
        List<DictTenantData> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject jsonObject = data.getJSONObject(i);
            String label = jsonObject.getString("status_name");
            String key = jsonObject.getString("status_id");
            boolean result = collect.containsKey(label);
            if (!result) {
                DictTenantData dictTenantData = new DictTenantData();
                dictTenantData.setDictCode("customer_status");
                dictTenantData.setDictValue(label);
                dictTenantData.setDictKey(key);
                dictTenantData.setTenantId(XiaomanContant.TENANT_ID);
                dictTenantData.setSort(100);
                list.add(dictTenantData);
            } else {
                DictTenantData dictTenantData = collect.get(label);
                dictTenantData.setDictKey(key);
                dictTenantDataService.updateById(dictTenantData);
            }
        }
        if (!list.isEmpty()) {
            dictTenantDataService.saveBatch(list);
        }
    }

    @Override
    public void dealCustomerTagDict(String label, String key) {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        DictTenantData dictTenantData = new DictTenantData();
        dictTenantData.setDictCode("customer_tag");
        dictTenantData.setDictValue(label);
        dictTenantData.setDictKey(key);
        dictTenantData.setTenantId(XiaomanContant.TENANT_ID);
        dictTenantData.setSort(100);
        dictTenantDataService.save(dictTenantData);
        DynamicDataSourceContextHolder.poll();

    }

    /**
     * 推送客户数据到小满
     */
    @DSTransactional
    @Override
    public void updateXiaomanData(CustomerDto dto) {
        if (ObjectUtil.isEmpty(dto.getCompanyId())) {
            return;
        }
        XiaomanUpdateInfoVO xVo = new XiaomanUpdateInfoVO();
        List<JSONObject> xiaomanCustomerUserList = new ArrayList<>();

        //=========================
        List<CustomerUser> customerUserList = dto.getCustomerUserList();
        for (CustomerUser cu : customerUserList) {
            String contactJsonStr = cu.getContactJson();
            log.info("->" + contactJsonStr + "<-");
            JSONArray contactJsonArr = ObjectUtil.isEmpty(contactJsonStr) ? new JSONArray() : JSONArray.parseArray(contactJsonStr);

            JSONObject xCU = new JSONObject();
            xCU.put("id", cu.getId());//联系人id
            xCU.put("mainCustomerFlag", 0);//主要联系人设置标志
            xCU.put("customerId", cu.getCustomerId());//客户id
            xCU.put("name", cu.getName());//客户联系人名称
            xCU.put("email", cu.getEmail());//客户联系人邮箱
            xCU.put("remark", dto.getRemark());//客户联系人备注
            xCU.put("telAreaCode", dto.getTelAreaCode());//客户联系人电话区号
            //更多联系方式
            for (int i = 0; i < contactJsonArr.size(); i++) {
                JSONObject contactJson = contactJsonArr.getJSONObject(i);
                xCU.put(contactJson.getString("type"), contactJson.getString("contactNo"));
            }
            xiaomanCustomerUserList.add(xCU);

//            XiaomanUpdateInfoVO.Customer xCU = new XiaomanUpdateInfoVO.Customer();
//            xCU.setName(cu.getName());//联系人名称
//            xCU.setEmail(cu.getEmail());//客户简称
//            xCU.setRemark(dto.getRemark());//客户联系人备注
//            xCU.setTelAreaCode(Long.parseLong(dto.getTelAreaCode())); //客户联系人电话区号
//
//            //更多联系方式
//            for (int i = 0; i < contactJsonArr.size(); i++) {
//                JSONObject contactJson = contactJsonArr.getJSONObject(i);
//                String contactNo = contactJson.getString("contactNo");
//                switch (contactJson.getString("type")) {
//                    case "mobile":
//                        xCU.setTel(contactJson.getLong("contactNo"));//客户联系人电话
//                        break;
//                    case "qq":
//                        xCU.setQq(contactNo);//客户联系人QQ
//                        break;
//                    case "LinkedIn":
//                        xCU.setLinkedin(contactNo);//客户联系人LinkedIn
//                        break;
//                    case "Skype":
//                        xCU.setSkype(contactNo);//客户联系人Skype
//                        break;
//                    case "wa":
//                        xCU.setWhatsapp(contactNo);//客户联系人Whatsapp
//                        break;
//                    case "Facebook":
//                        xCU.setFacebook(contactNo);//客户联系人Facebook
//                        break;
//                    case "wechat":
//                        xCU.setWechat(contactNo);//客户联系人Wechat
//                        break;
//                    case "twitter":
//                        xCU.setTwitter(contactNo);//客户联系人Twitter
//                        break;
//                }
//            }
//            xiaomanCustomerUserList.add(xCU);
        }
        //=========================
        xVo.setCompanyId(dto.getCompanyId());//公司客户ID (创建时无需设置，更新时需要设置)
        xVo.setSerialId(dto.getCustomerCode());//客户编号id->客户表customer_code
        xVo.setName(dto.getName());//客户名称
        xVo.setShortName(dto.getShortName());//客户简称
        xVo.setHomepage(dto.getHomepage());//网站主页
        xVo.setFax(ObjectUtil.isEmpty(dto.getFax()) ? null : Long.parseLong(dto.getFax()));//传真
        xVo.setTelAreaCode(ObjectUtil.isEmpty(dto.getTelAreaCode()) ? null : Long.parseLong(dto.getTelAreaCode()));//客户电话区号
        xVo.setTel(ObjectUtil.isEmpty(dto.getTel()) ? null : Long.parseLong(dto.getTel()));//客户电话
        xVo.setAddress(dto.getAddress());//地址
        xVo.setRemark(dto.getRemark());//备注
        xVo.setCustomers(xiaomanCustomerUserList);//客户联系人
        //客户来源ID
        List<Long> originList = Arrays.stream(dto.getSource().split(",")).map(Long::valueOf).collect(Collectors.toList());
        xVo.setOriginList(originList);
        xVo.setTrailStatus(ObjectUtil.isEmpty(dto.getStatus()) ? null : Long.parseLong(dto.getStatus()));//客户状态ID
        //=========================

        updateXiaomanData(xVo);
    }
}
